@@ -1,4 +1,4 @@
-/* $Id: imapcommands.c,v 1.111 2004/01/04 19:33:44 ilja Exp $
+/* $Id: imapcommands.c,v 1.111.2.1 2004/09/07 09:54:28 ilja Exp $
  * (c) 2000-2002 IC&S, The Netherlands
  *
  * imapcommands.c
@@ -1844,7 +1844,7 @@ int _ic_fetch(char *tag, char **args, ClientInfo *ci)
 	}
     }
 
-  fetch_list.start = list_reverse(fetch_list.start);
+  fetch_list.start = dbmail_list_reverse(fetch_list.start);
 
   /* now fetch results for each msg */
   endptr = args[0];
