@@ -1,4 +1,4 @@
-/* $Id: list.h,v 1.13 2003/03/17 16:04:08 roel Exp $
+/* $Id: list.h,v 1.13.6.1 2004/09/07 09:54:28 ilja Exp $
  * (c) 2000-2002 IC&S, The Netherlands
  *
  * list.h: list header */
@@ -40,6 +40,9 @@ void list_freelist(struct element **start);
 long list_totalnodes(struct list *tlist);
 void list_showlist(struct list *tlist);
 void list_init(struct list *tlist);
-struct element* list_reverse(struct element *start);
+/* this function is renamed to dbmail_list_reverse because MySQL 4.1
+ * already exports a function named list_reverse(). A bug report about
+ * that was filed at bugs.mysql.com :) */
+struct element* dbmail_list_reverse(struct element *start);
 
 #endif 
