@@ -1,4 +1,4 @@
-/*	$Id: misc.c,v 1.9 2003/10/24 09:21:30 ilja Exp $
+/*	$Id: misc.c,v 1.9.2.1 2004/09/23 13:23:23 ilja Exp $
  *	(c) 2000-2002 IC&S, The Netherlands
  *
  *	Miscelaneous functions */
@@ -56,11 +56,22 @@ char *itoa(int i)
 
 void create_unique_id(char *target, u64_t messageid)
 {
-  if (messageid != 0)
-    snprintf (target,UID_SIZE,"%s:%s",itoa(messageid),itoa(rand()));
-  else
-    snprintf (target,UID_SIZE,"%s",itoa(rand()));
-  snprintf (target,UID_SIZE,"%s",makemd5(target));
+  char *md5_str;
+  char *messageid_str;
+  char *rand_str;
+    
+  rand_str = itoa(rand());
+  if (messageid != 0) {
+    messageid_str = itoa(messageid);
+    snprintf (target,UID_SIZE,"%s:%s", messageid_str, rand_str);
+    my_free(messageid_str);
+  } else {
+    snprintf (target,UID_SIZE,"%s", rand_str);
+  }
+  my_free(rand_str);
+  md5_str = makemd5(target);
+  snprintf (target,UID_SIZE,"%s", md5_str);
+  my_free(md5_str);
   trace (TRACE_DEBUG,"create_unique_id(): created: %s",target);
 }
 
