@@ -1,5 +1,5 @@
 /*
- * $Id: dbauthmysql.c,v 1.30.2.4 2004/06/14 14:13:00 ilja Exp $
+ * $Id: dbauthmysql.c,v 1.30.2.5 2004/09/23 14:49:54 ilja Exp $
  * (c) 2000-2002 IC&S, The Netherlands (http://www.ic-s.nl)
  * implementation of authentication functions &
  * user management for mySQL dbases
@@ -650,6 +650,7 @@ u64_t auth_validate (char *user, char *password)
   char salt[13];
   char cryptres[35];
   char *escuser;
+  char *md5_str;
 
   if (user == NULL || password == NULL) {
 	  trace(TRACE_DEBUG, "auth_validate(): user or password is NULL");
@@ -708,7 +709,9 @@ u64_t auth_validate (char *user, char *password)
       if ( strncmp(__auth_row[1], "$1$", 3) ) 
 	{
 	  trace (TRACE_DEBUG,"auth_validate(): validating using MD5 digest comparison");
-	  is_validated = (strncmp( makemd5(password), __auth_row[1], 32) == 0) ? 1 : 0;
+	  md5_str = makemd5(password);
+	  is_validated = (strncmp( md5_str, __auth_row[1], 32) == 0) ? 1 : 0;
+	  my_free(md5_str);
 	}
       else
 	{
@@ -727,15 +730,16 @@ u64_t auth_validate (char *user, char *password)
   else if ( strcasecmp(__auth_row[2], "md5sum") == 0)
     {
       trace (TRACE_DEBUG,"auth_validate(): validating using MD5 digest comparison");
-      is_validated = (strncmp( makemd5(password), __auth_row[1], 32) == 0) ? 1 : 0;
+      md5_str = makemd5(password);
+      is_validated = (strncmp( md5_str, __auth_row[1], 32) == 0) ? 1 : 0;
+      my_free(md5_str);
     }
                        
-  
+  id = (__auth_row[0]) ? strtoull(__auth_row[0], NULL, 10) : 0;  
+  mysql_free_result(__auth_res);
   if (is_validated)
     {
-      id = (__auth_row[0]) ? strtoull(__auth_row[0], NULL, 10) : 0;
-
-       /* log login in the dbase */
+      /* log login in the dbase */
       snprintf(__auth_query_data, AUTH_QUERY_SIZE, "UPDATE users SET last_login = '%s' "
 	       "WHERE user_idnr = %llu", timestr, id);
       
@@ -745,8 +749,6 @@ u64_t auth_validate (char *user, char *password)
   else
     id = 0;
 
-  mysql_free_result(__auth_res);
-
   return id;
 }
 
@@ -756,6 +758,7 @@ u64_t auth_md5_validate (char *username,unsigned char *md5_apop_he, char *apop_s
   
   char *checkstring;
   unsigned char *md5_apop_we;
+  char *checkstring_md5;
   u64_t useridnr;	
   char timestr[30];
   time_t td;
@@ -806,20 +809,21 @@ u64_t auth_md5_validate (char *username,unsigned char *md5_apop_he, char *apop_s
   memtst((checkstring=(char *)my_malloc(strlen(apop_stamp)+strlen(__auth_row[0])+2))==NULL);
   snprintf(checkstring, strlen(apop_stamp)+strlen(__auth_row[0])+2, "%s%s",apop_stamp,__auth_row[0]);
 
-  md5_apop_we=makemd5(checkstring);
+  md5_apop_we = makemd5(checkstring);
 	
   trace(TRACE_DEBUG,"auth_md5_validate(): checkstring for md5 [%s] -> result [%s]",checkstring,
 	md5_apop_we);
   trace(TRACE_DEBUG,"auth_md5_validate(): validating md5_apop_we=[%s] md5_apop_he=[%s]",
 	md5_apop_we, md5_apop_he);
 
-  if (strcmp(md5_apop_he,makemd5(checkstring))==0)
+  if (strcmp(md5_apop_he, md5_apop_we)==0)
     {
       trace(TRACE_MESSAGE,"auth_md5_validate(): user [%s] is validated using APOP",username);
 		
       useridnr = (__auth_row && __auth_row[1]) ? strtoull(__auth_row[1], NULL, 10) : 0;
       mysql_free_result(__auth_res);
       my_free(checkstring);
+      my_free(md5_apop_we);
 
        /* log login in the dbase */
       snprintf(__auth_query_data, AUTH_QUERY_SIZE, "UPDATE users SET last_login = '%s' "
@@ -838,6 +842,7 @@ u64_t auth_md5_validate (char *username,unsigned char *md5_apop_he, char *apop_s
   
   
   my_free(checkstring);
+  my_free(md5_apop_we);
   
   return 0;
 }
