@@ -1,5 +1,5 @@
 /*
- * $Id: dbauthpgsql.c,v 1.47.2.3 2004/06/04 09:04:06 ilja Exp $
+ * $Id: dbauthpgsql.c,v 1.47.2.4 2004/09/23 14:49:54 ilja Exp $
  *
  * implementation of authentication functions &
  * user management for PostgreSQL dbases
@@ -724,6 +724,7 @@ u64_t auth_validate (char *user, char *password)
   char salt[13];
   char cryptres[35];
   char *escuser;
+  char *md5_str;
 
   if (user == NULL || password == NULL) {
 	  trace(TRACE_DEBUG, "auth_validate(): user or password is NULL");
@@ -778,7 +779,9 @@ u64_t auth_validate (char *user, char *password)
        {
          trace (TRACE_DEBUG,"auth_validate(): validating using MD5 digest comparison");
 	 row = PQgetvalue(__auth_res, 0, 1);
-	 is_validated = (strncmp( makemd5(password), row, 32) == 0) ? 1 : 0;
+	 md5_str = makemd5(password);
+	 is_validated = (strncmp( md5_str, row, 32) == 0) ? 1 : 0;
+	 my_free(md5_str);
        }
       else
        {
@@ -798,7 +801,9 @@ u64_t auth_validate (char *user, char *password)
     {
       trace (TRACE_DEBUG,"auth_validate(): validating using MD5 digest comparison");
       row = PQgetvalue(__auth_res, 0, 1);
-      is_validated = (strncmp( makemd5(password), row, 32) == 0) ? 1 : 0;
+      md5_str = makemd5(password);
+      is_validated = (strncmp( md5_str, row, 32) == 0) ? 1 : 0;
+      my_free(md5_str);
     }
 
   if (is_validated)
@@ -878,14 +883,14 @@ u64_t auth_md5_validate (char *username,unsigned char *md5_apop_he, char *apop_s
   memtst((checkstring=(char *)my_malloc(strlen(apop_stamp)+strlen(value)+2))==NULL);
   snprintf(checkstring, strlen(apop_stamp)+strlen(value)+2, "%s%s",apop_stamp,value);
 
-  md5_apop_we=makemd5(checkstring);
+  md5_apop_we = makemd5(checkstring);
 	
   trace(TRACE_DEBUG,"auth_md5_validate(): checkstring for md5 [%s] -> result [%s]",checkstring,
 	md5_apop_we);
   trace(TRACE_DEBUG,"auth_md5_validate(): validating md5_apop_we=[%s] md5_apop_he=[%s]",
 	md5_apop_we, md5_apop_he);
 
-  if (strcmp(md5_apop_he,makemd5(checkstring))==0)
+  if (strcmp(md5_apop_he, md5_apop_we)==0)
     {
       trace(TRACE_MESSAGE,"auth_md5_validate(): user [%s] is validated using APOP",username);
 		
@@ -893,6 +898,7 @@ u64_t auth_md5_validate (char *username,unsigned char *md5_apop_he, char *apop_s
       useridnr = (value) ? strtoull(value, NULL, 10) : 0;
       PQclear(__auth_res);
       my_free(checkstring);
+      my_free(md5_apop_we);
 
        /* log login in the dbase */
       snprintf(__auth_query_data, AUTH_QUERY_SIZE, "UPDATE users SET last_login = '%s' "
@@ -909,8 +915,8 @@ u64_t auth_md5_validate (char *username,unsigned char *md5_apop_he, char *apop_s
   if (__auth_res)
     PQclear(__auth_res);
   
-  
   my_free(checkstring);
+  my_free(md5_apop_we);
   
   return 0;
 }
