@@ -1,4 +1,4 @@
-/* $Id: pipe.c,v 1.97.2.2 2004/06/08 14:01:35 ilja Exp $
+/* $Id: pipe.c,v 1.97.2.3 2004/09/15 14:21:09 ilja Exp $
  * (c) 2000-2002 IC&S, The Netherlands 
  *
  * Functions for reading the pipe from the MTA */
@@ -130,7 +130,7 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
   /* 	this loop gets all the users from the list 
 	and check if they're in the database */
 
-  struct element *tmp,*ret_path;
+  struct element *tmp, *ret_path;
   char *insertquery;
   char *updatequery;
   char *unique_id;
@@ -144,7 +144,7 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
   struct list messageids;
   struct list external_forwards;
   struct list bounces;
-  u64_t temp_message_record_id,userid, bounce_userid;
+  u64_t temp_message_record_id, userid, bounce_userid;
   int i, this_user;
   int do_auto_notify = 0, do_auto_reply = 0;
   char *reply_body, *notify_address;
@@ -287,7 +287,7 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
 	{
 	  /* FIXME: it's probably a forward to another address
 	   * we need to do an email address validity test if the first char !| */
-	  trace (TRACE_DEBUG,"insert_messages(): no numeric value in deliver_to, "
+	  trace (TRACE_DEBUG,"insert_messages(): non-numeric value in deliver_to, "
 		 "calling external_forward");
 
 	  /* creating a list of external forward addresses */
@@ -329,11 +329,14 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
   /* reading rest of the pipe and creating messageblocks 
    * we need to create a messageblk for each messageid */
 
-  trace (TRACE_DEBUG,"insert_messages(): allocating [%d] bytes of memory for readblock",READ_BLOCK_SIZE);
+  trace (TRACE_DEBUG,"insert_messages(): allocating [%d] "
+	"bytes of memory for readblock",READ_BLOCK_SIZE);
 
   memtst ((strblock = (char *)my_malloc(READ_BLOCK_SIZE+1))==NULL);
 	
-	/* first we need to check if we need to deliver into the database */
+  /* first check if we need to deliver into the database
+   * and if so, insert message blocks */
+
   if (list_totalnodes(&messageids)>0)
     {
       totalmem = 0; /* reset totalmem counter */
@@ -348,6 +351,7 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
 	      trace(TRACE_ERROR,"insert_messages(): error on instream: [%s]", strerror(errno));
 	    }
 
+	  /* FIXME: this violates rfc 2822, which allows nulls in a message */
 	  /* replace all errorneous '\0' by ' ' (space) */
 	  for (i=0; i<usedmem; i++)
 	    {
@@ -381,13 +385,12 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
 	  else 
 	    trace (TRACE_DEBUG, "insert_messages(): end of instream stream");
 		
-	
 	}
 		
       trace (TRACE_DEBUG,"insert_messages(): updating size fields");
 	
 
-      /* we need to update messagesize in all messages */
+      /* update messagesize in all messages */
       tmp=list_getstart(&messageids);
       while (tmp!=NULL)
 	{
@@ -397,8 +400,42 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
 	   * even more unique strings */
 	  create_unique_id(unique_id,*(u64_t*)tmp->data); 
 	  db_update_message (*(u64_t*)tmp->data,unique_id,totalmem+headersize,0);
+	  tmp=tmp->nextnode;
+	}
+    }
 
-	  /* checking size */
+  /* next do any external forwards */
+  if (list_totalnodes(&external_forwards)>0)
+    {
+      /* sending the message to forwards */
+  
+      trace (TRACE_DEBUG,"insert_messages(): delivering to external addresses");
+  
+      ret_path = list_getstart(returnpath);
+      
+      if (list_totalnodes(&messageids)==0)
+	{
+	  /* deliver using stdin */
+	  pipe_forward (stdin, &external_forwards,
+		ret_path ? ret_path->data : "DBMAIL-MAILER", header, 0);
+	}
+      else
+	{
+	  /* deliver using database */
+	  tmp = list_getstart(&messageids);
+	  pipe_forward (stdin, &external_forwards,
+		ret_path ? ret_path->data: "DBMAIL-MAILER", header, 
+		*((u64_t *)tmp->data));
+	}
+    }
+
+  /* now do quoata checks and notifies/replies if ok */
+  if (list_totalnodes(&messageids)>0)
+    {
+      tmp=list_getstart(&messageids);
+      while (tmp!=NULL)
+	{
+	  /* check for users going over quota */
 	  switch (db_check_sizelimit (totalmem+headersize, *(u64_t*)tmp->data,
 				      &bounce_userid))
 	    {
@@ -464,6 +501,7 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
 		    trace(TRACE_ERROR, "insert_messages(): error fetching reply body");
 		  else
 		    {
+		      /* looks like a one-byte memory leak here if == '\0' */
 		      if (reply_body == NULL || reply_body[0] == '\0')
 			trace(TRACE_DEBUG, "insert_messages(): no reply body specified, skipping auto-reply");
 		      else
@@ -473,13 +511,10 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
 			}
 		    }
 		}
-		  
 	      
 	      break;
 	    }
 
-
-	  temp_message_record_id=*(u64_t*)tmp->data;
 	  tmp=tmp->nextnode;
 	}
     }
@@ -497,29 +532,6 @@ int insert_messages(char *header, u64_t headersize, struct list *users,
       }
   }
 
-  /* do we have forward addresses ? */
-  if (list_totalnodes(&external_forwards)>0)
-    {
-      /* sending the message to forwards */
-  
-      trace (TRACE_DEBUG,"insert_messages(): delivering to external addresses");
-  
-      ret_path = list_getstart(returnpath);
-      
-      if (list_totalnodes(&messageids)==0)
-	{
-	  /* deliver using stdin */
-	  pipe_forward (stdin, &external_forwards, ret_path ? ret_path->data : "DBMAIL-MAILER", header, 0);
-	}
-      else
-	{
-	  /* deliver using database */
-	  tmp = list_getstart(&messageids);
-	  pipe_forward (stdin, &external_forwards, ret_path ? ret_path->data: "DBMAIL-MAILER", header, 
-			*((u64_t *)tmp->data));
-	}
-    }
-	
   trace (TRACE_DEBUG,"insert_messages(): Freeing memory blocks");
 
   /* memory cleanup */
