@@ -1,4 +1,4 @@
-/* $Id: pop3.c,v 1.64 2003/10/24 09:21:30 ilja Exp $
+/* $Id: pop3.c,v 1.64.2.1 2004/09/23 15:42:45 ilja Exp $
  * (c) 2000-2002 IC&S, The Netherlands
  *
  * implementation for pop3 commands according to RFC 1081 */
@@ -201,6 +201,9 @@ int pop3_handle_connection (clientinfo_t *ci)
 	else
 		trace (TRACE_ERROR, "pop3_handle_connection(): error, uncomplete session");
 
+	/* cleanup session. Removes message list etc */
+	db_session_cleanup(&session);
+
 	if (session.username != NULL)
 	{
 		/* username cleanup */
