@@ -1,5 +1,5 @@
 /*
- * $Id: rfcmsg.c,v 1.6 2003/10/24 09:21:30 ilja Exp $
+ * $Id: rfcmsg.c,v 1.6.2.1 2004/09/07 09:54:28 ilja Exp $
  * (c) 2000-2002 IC&S, The Netherlands (http://www.ic-s.nl)
  * function implementations for parsing an RFC822/MIME 
  * compliant mail message
@@ -74,11 +74,11 @@ void db_reverse_msg(mime_message_t *msg)
     }
 
   /* reverse this list */
-  msg->children.start = list_reverse(msg->children.start);
+  msg->children.start = dbmail_list_reverse(msg->children.start);
 
   /* reverse header items */
-  msg->mimeheader.start = list_reverse(msg->mimeheader.start);
-  msg->rfcheader.start  = list_reverse(msg->rfcheader.start);
+  msg->mimeheader.start = dbmail_list_reverse(msg->mimeheader.start);
+  msg->rfcheader.start  = dbmail_list_reverse(msg->rfcheader.start);
 }
 
 /*
